from gruffy.base import *
from gruffy.sidebar import SideBar


class StackedSideBar(SideBar, StackedMixin):
    """Stacked Side Bar Graph Object"""

    bar_spacing = None

    def draw(self):
        self.has_left_labels = True
        self.get_maximum_by_stack()
        SideBar.draw(self)

    def draw_bars(self):
        # Setup spacing.
        #
        # Columns sit stacked.
        self.bar_spacing = self.bar_spacing or 0.9

        self.bar_width = self.graph_height / float(self.column_count)
        dl = DrawableList()
        dl.append(DrawableStrokeOpacity(0.0))
        height = [0 for i in range(self.column_count)]
        length = [self.graph_left for i in range(self.column_count)]
        padding = (self.bar_width * (1 - self.bar_spacing)) / 2

        for row_index, data_row in enumerate(self.norm_data):
            for point_index, data_point in enumerate(data_row['values']):
                # using the original calcs from the stacked bar chart to
                # get the difference between part of the bart chart
                # we wish to stack.
                tmp1 = self.graph_left + (self.graph_width - \
                       data_point * self.graph_width - height[point_index]) + 1
                tmp2 = self.graph_left + \
                       self.graph_width - height[point_index] - 1
                difference = tmp2 - tmp1
                dl.append(DrawableFillColor(Color(data_row['color'])))
                if type(self.transparent) is float:
                    dl.append(DrawableFillOpacity(self.transparent))
                elif self.transparent is True:
                    dl.append(DrawableFillOpacity(DEFAULT_TRANSPARENCY))
                left_x = length[point_index]
                left_y = self.graph_top + \
                         (self.bar_width * point_index) + padding
                right_x = left_x + difference
                right_y = left_y + self.bar_width * self.bar_spacing
                length[point_index] += difference
                height[point_index] += (data_point * self.graph_width - 2)

                dl.append(DrawableRectangle(left_x, left_y, right_x, right_y))

                # Calculate center based on bar_width and current row
                label_center = self.graph_top + \
                               (self.bar_width * point_index) + \
                               (self.bar_width * self.bar_spacing / 2.0)
                self.draw_label(label_center, point_index)
        dl.append(DrawableScaling(self.scale, self.scale))
        self.base_image.draw(dl)

    def larger_than_max(self, data_point, index=0):
        if self.lmax(data_point, index) > self.maximum_value:
            return True
        return False

    def lmax(self, data_point, index):
        #self.gdata.inject(0) {|sum, item| sum + item['values'][index]}
        total = 0
        for data in self.gdata:
            total += data['values'][index]
        return total



from gruffy.base import *


class StackedBar(Base, StackedMixin):
    """Stacked Bar Graph Object"""

    bar_spacing = None

    def draw(self):
        self.get_maximum_by_stack()
        StackedBar.__base__.draw(self)
        if not self.has_gdata:
            return

        # Setup spacing.
        #
        # Columns sit stacked.
        self.bar_spacing = self.bar_spacing or 0.9
        self.bar_width = self.graph_width / float(self.column_count)
        padding = (self.bar_width * (1 - self.bar_spacing)) / 2

        dl = DrawableList()
        dl.append(DrawableStrokeOpacity(0.0))
        height = [0 for i in range(self.column_count)]
        for raw_index, data_row in enumerate(self.norm_data):
            for point_index, data_point in enumerate(data_row['values']):
                dl.append(DrawableFillColor(Color(data_row['color'])))
                # Calculate center based on bar_width and current row
                label_center = self.graph_left + \
                               (self.bar_width * point_index) + \
                               (self.bar_width * self.bar_spacing / 2.0)
                self.draw_label(label_center, point_index)

                if data_point == 0:
                    continue
                # Use incremented x and scaled y
                left_x = self.graph_left + \
                         (self.bar_width * point_index) + padding
                left_y = self.graph_top + (self.graph_height - \
                         data_point * self.graph_height - \
                         height[point_index]) + 1
                right_x = left_x + self.bar_width * self.bar_spacing
                right_y = self.graph_top + \
                          self.graph_height - height[point_index] - 1
                # update the total height of the current stacked bar
                height[point_index] += (data_point * self.graph_height)
                dl.append(DrawableRectangle(left_x, left_y, right_x, right_y))
        dl.append(DrawableScaling(self.scale, self.scale))
        self.base_image.draw(dl)



from gruffy.base import *


class Dot(Base):
    """Dot Graph Object"""

    def draw(self):
        self.has_left_labels = True
        Dot.__base__.draw(self)
        if not self.has_gdata:
            return

        spacing_factor = 1.0

        self.items_width = self.graph_height / float(self.column_count)
        self.item_width = self.items_width * \
                          spacing_factor / len(self.norm_data)

        dl = DrawableList()
        dl.append(DrawableStrokeOpacity(0.0))
        padding = (self.items_width * (1 - spacing_factor)) / 2

        for row_index, data_row in enumerate(self.norm_data):
            for point_index, data_point in enumerate(data_row['values']):
                x_pos = self.graph_left + (data_point * self.graph_width) - \
                        round(float(self.item_width) / 6.0)
                y_pos = self.graph_top + (self.items_width * point_index) + \
                        padding + round(float(self.item_width) / 2.0)
                if row_index == 0:
                    dl.append(DrawableStrokeColor(Color(self.marker_color)))
                    dl.append(DrawableStrokeWidth(1.0))
                    dl.append(DrawableStrokeOpacity(0.1))
                    dl.append(DrawableLine(self.graph_left,
                                           y_pos,
                                           self.graph_left + self.graph_width,
                                           y_pos))
                dl.append(DrawableFillColor(Color(data_row['color'])))
                if type(self.transparent) is float:
                    dl.append(DrawableFillOpacity(self.transparent))
                elif self.transparent is True:
                    dl.append(DrawableFillOpacity(DEFAULT_TRANSPARENCY))
                dl.append(DrawableStrokeColor('transparent'))
                dl.append(DrawableCircle(x_pos,
                    y_pos, x_pos + round(float(self.item_width) / 3.0), y_pos))
                # Calculate center based on item_width and current row
                label_center = self.graph_top + (self.items_width * \
                        point_index + self.items_width / 2) + padding
                self.draw_label(label_center, point_index)
        dl.append(DrawableScaling(self.scale, self.scale))
        self.base_image.draw(dl)

    def draw_line_markers(self):
        if self.hide_line_markers:
            return
        dl = DrawableList()
        dl.append(DrawableStrokeAntialias(False))

        # Draw horizontal line markers and annotate with numbers
        dl.append(DrawableStrokeColor(Color(self.marker_color)))
        dl.append(DrawableStrokeWidth(1))
        number_of_lines = 5

        # TODO Round maximum marker value to a round number like
        # 100, 0.1, 0.5, etc.
        increment = self.significant(float(self.maximum_value) / number_of_lines)
        for index in range(number_of_lines + 1):
            line_diff = (self.graph_right - self.graph_left) / number_of_lines
            x = self.graph_right - (line_diff * index) - 1
            dl.append(DrawableLine(x, self.graph_bottom,
                                   x, self.graph_bottom + 0.5 * LABEL_MARGIN))
            diff = index - number_of_lines
            marker_label = abs(diff) * increment

            if not self.hide_line_numbers:
                dl.append(DrawableFillColor(Color(self.font_color)))
                font = self.font if self.font else DEFAULT_FONT
                dl.append(DrawableFont(font, StyleType.NormalStyle, 400,
                                       StretchType.NormalStretch))
                dl.append(DrawableStrokeColor('transparent'))
                dl.append(DrawablePointSize(self.marker_font_size))
                dl.append(DrawableGravity(GravityType.NorthWestGravity))
                font_hight = self.calculate_caps_height(self.marker_font_size)
                text_width = self.calculate_width(self.marker_font_size,
                                                  str(marker_label))
                x -= text_width / 2.0
                y = font_hight / 2.0 + self.graph_bottom + (LABEL_MARGIN * 2.0)
                dl.append(DrawableText(x, y, str(marker_label)))
            dl.append(DrawableStrokeAntialias(True))
        dl.append(DrawableScaling(self.scale, self.scale))
        self.base_image.draw(dl)

    def draw_label(self, y_offset, index):
        if index in self.labels and index not in self.labels_seen:
            dl = DrawableList()
            dl.append(DrawableFillColor(Color(self.font_color)))
            font = self.font if self.font else DEFAULT_FONT
            dl.append(DrawableFont(font, StyleType.NormalStyle, 400,
                                   StretchType.NormalStretch))
            dl.append(DrawableStrokeColor('transparent'))
            dl.append(DrawablePointSize(self.marker_font_size))
            dl.append(DrawableGravity(GravityType.NorthGravity))
            font_hight = self.calculate_caps_height(self.marker_font_size)
            text_width = self.calculate_width(self.marker_font_size,
                                              self.labels[index])
            x = -(self.graph_left + LABEL_MARGIN * 2.7 + text_width / 2.0)
            y = y_offset + font_hight / 2.0
            dl.append(DrawableText(x, y, self.labels[index]))
            self.labels_seen[index] = 1
            dl.append(DrawableScaling(self.scale, self.scale))
            self.base_image.draw(dl)



from gruffy.area import Area
from gruffy.bar import Bar
from gruffy.dot import Dot
from gruffy.line import Line
from gruffy.pie import Pie
from gruffy.sidebar import SideBar
from gruffy.stacked_area import StackedArea
from gruffy.stacked_bar import StackedBar
from gruffy.stacked_sidebar import StackedSideBar

__all__ = ['Area', 'Bar', 'Dot', 'Line', 'Pie', 'SideBar', 'StackedArea', \
           'StackedBar', 'StackedSideBar']



import copy
from gruffy.base import *


class StackedArea(Base, StackedMixin):
    """Stacked Area Graph Object"""

    last_series_goes_on_bottom = None

    def draw(self):
        self.get_maximum_by_stack()
        StackedArea.__base__.draw(self)
        if not self.has_gdata:
            return
        dl = DrawableList()
        self.x_increment = self.graph_width / float(self.column_count - 1)
        dl.append(DrawableStrokeColor('transparent'))

        height = [0 for i in range(self.column_count)]

        data_points = None
        if self.last_series_goes_on_bottom:
            self.norm_data.reverse()
        for data_row in self.norm_data:
            prev_data_points = data_points
            data_points = []
            dl.append(DrawableFillColor(Color(data_row['color'])))
            for index, data_point in enumerate(data_row['values']):
                # Use incremented x and scaled y
                new_x = self.graph_left + (self.x_increment * index)
                new_y = self.graph_top + (self.graph_height - data_point * \
                        self.graph_height - height[index])

                height[index] += (data_point * self.graph_height)
                data_points.append(new_x)
                data_points.append(new_y)
                self.draw_label(new_x, index)

            if prev_data_points:
                poly_points = copy.copy(data_points)
                tmp_max_point = len(prev_data_points) / 2 - 1
                for i in range(tmp_max_point, -1, -1):
                    poly_points.append(prev_data_points[2 * i])
                    poly_points.append(prev_data_points[2 * i + 1])
                poly_points.append(data_points[0])
                poly_points.append(data_points[1])
            else:
                poly_points = copy.copy(data_points)
                poly_points.append(self.graph_right)
                poly_points.append(self.graph_bottom - 1)
                poly_points.append(self.graph_left)
                poly_points.append(self.graph_bottom - 1)
                poly_points.append(data_points[0])
                poly_points.append(data_points[1])
            # generate CoordinateList
            coordinates = [Coordinate(poly_points[i], poly_points[i + 1]) \
                           for i in range(0, len(poly_points), 2)]
            cl = CoordinateList()
            for c in coordinates:
                cl.append(c)
            dl.append(DrawablePolyline(cl))
        dl.append(DrawableScaling(self.scale, self.scale))
        self.base_image.draw(dl)



from gruffy.base import *


class Line(Base):
    """Line Graph Object"""

    line_width = None
    dot_radius = None
    hide_lines = None
    hide_dots = None

    def __init__(self, *args):
        if len(args):
            Base.__init__(self, args[0])
        else:
            Base.__init__(self)
        self.hide_dots = self.hide_lines = False
        self.baseline_color = 'red'
        self.baseline_value = None

    def draw(self):
        Line.__base__.draw(self)
        if not self.has_gdata:
            return
        if self.column_count > 1:
            x_increment = float(self.graph_width / (self.column_count - 1))
        else:
            x_increment = self.graph_width

        dl = DrawableList()
        if hasattr(self, "norm_baseline"):
            level = self.graph_top + (self.graph_height - \
                    self.norm_baseline * self.graph_height)
            dl.append(DrawableStrokeColor(Color(self.baseline_color)))
            dl.append(DrawableFillOpacity(0.0))
            dl.append(DrawableStrokeDasharray(10, 20))
            dl.append(DrawableStrokeWidth(5))
            dl.append(DrawableLine(self.graph_left,
                level, self.graph_left + self.graph_width, level))
            self.base_image.draw(dl)
            del(dl)

        for data_row in self.norm_data:
            prev_x = prev_y = None
            self.one_point = self.is_contains_one_point_only(data_row)
            for index, data_point in enumerate(data_row['values']):
                new_x = self.graph_left + (x_increment * index)
                if data_point is None:
                    continue
                self.draw_label(new_x, index)
                new_y = self.graph_top + (self.graph_height - \
                        data_point * self.graph_height)

                # Reset each time to avoid thin-line errors
                dl.append(DrawableStrokeColor(Color(data_row['color'])))
                dl.append(DrawableFillColor(Color(data_row['color'])))
                if type(self.transparent) is float:
                    dl.append(DrawableFillOpacity(self.transparent))
                elif self.transparent is True:
                    dl.append(DrawableFillOpacity(DEFAULT_TRANSPARENCY))
                dl.append(DrawableStrokeOpacity(1.0))
                dl.append(DrawableStrokeWidth(self.line_width or \
                        self.clip_value_if_greater_than(self.columns / \
                        (len(self.norm_data[0]['values']) * 4), 5.0)))

                circle_radius = self.dot_radius or \
                        self.clip_value_if_greater_than(self.columns / \
                        (len(self.norm_data[0]['values']) * 2.5), 5.0)

                if not self.hide_lines and prev_x and prev_y:
                    dl.append(DrawableLine(prev_x, prev_y, new_x, new_y))
                elif self.one_point:
                    dl.append(DrawableCircle(new_x, new_y,
                                             new_x - circle_radius, new_y))
                if not self.hide_dots:
                    dl.append(DrawableCircle(new_x, new_y,
                                             new_x - circle_radius, new_y))

                prev_x = new_x
                prev_y = new_y
        dl.append(DrawableScaling(self.scale, self.scale))
        self.base_image.draw(dl)

    def normalize(self):
        if self.baseline_value:
            tmp = float(self.baseline_value)
        else:
            tmp = 0
        self.maximum_value = max([float(self.maximum_value), tmp])
        Line.__base__.normalize(self)
        if self.baseline_value:
            self.norm_baseline = (float(self.baseline_value) / \
                                  float(self.maximum_value))

    def is_contains_one_point_only(self, data_row):
        # Spin through data to determine if there is just one_value present.
        one_point = False
        for data_point in data_row['values']:
            if data_point:
                if one_point:
                    # more than one point, bail
                    return False
                else:
                    # there is at least one data point
                    return True
        return one_point



from gruffy.base import *


class Area(Base):
    """Area Graph Object
    """

    def draw(self):
        """override to draw() method in Base Class."""
        Area.__base__.draw(self)
        if not self.has_gdata:
            return
        x_increment = self.graph_width / float(self.column_count - 1)
        dl = DrawableList()
        dl.append(DrawableStrokeColor(Color('transparent')))
        for data_row in self.norm_data:
            poly_points = CoordinateList()
            prev_x = prev_y = 0.0
            dl.append(DrawableFillColor(Color(data_row['color'])))
            if type(self.transparent) is float:
                dl.append(DrawableFillOpacity(self.transparent))
            elif self.transparent is True:
                dl.append(DrawableFillOpacity(DEFAULT_TRANSPARENCY))

            for index, data_point in enumerate(data_row['values']):
                # Use incremented x and scaled y
                new_x = self.graph_left + (x_increment * index)
                new_y = self.graph_top + \
                        (self.graph_height - data_point * self.graph_height)
                if prev_x > 0 and prev_y > 0:
                    poly_points.append(Coordinate(new_x, new_y))
                else:
                    poly_points.append(Coordinate(self.graph_left,
                                                  self.graph_bottom - 1))
                    poly_points.append(Coordinate(new_x, new_y))
                self.draw_label(new_x, index)
                prev_x = new_x
                prev_y = new_y
            # Add closing points, draw polygon
            poly_points.append(Coordinate(self.graph_right,
                                          self.graph_bottom - 1))
            poly_points.append(Coordinate(self.graph_left,
                                          self.graph_bottom - 1))
            dl.append(DrawablePolyline(poly_points))
        dl.append(DrawableScaling(self.scale, self.scale))
        self.base_image.draw(dl)



from gruffy.base import Base

g = Base()
g.theme_keynote()
g.theme_django()
g.theme_pastel()
g.theme_greyscale()
g.theme_37signals()
g.theme_rails_keynote()
g.theme_odeo()

g.data("test1", [1, 2, 3])
g.data("test2", [3, 2, 1])
g.write()
